@@ -72,7 +72,10 @@ PrimeConnection::~PrimeConnection ()
     std::vector<PrimeConnection *>::iterator it;
     for (it = connection_list.begin (); it != connection_list.end (); it++) {
         if ((*it) == this)
+	{
             connection_list.erase (it);
+	    break;
+	}
     }
 }
 
@@ -237,7 +240,6 @@ PrimeConnection::close_connection (void)
             case EBADF:
             case EINVAL:
             case EPIPE:
-            case EIO:
                 remaining = 0;
                 if (m_err_msg.empty ())
                     set_error_message (COMMUNICATION_FAILD, errno);
@@ -552,7 +554,6 @@ PrimeConnection::send_command (const char *command,
         case EBADF:
         case EINVAL:
         case EPIPE:
-        case EIO:
             clean_child ();
             if (m_err_msg.empty ())
                 set_error_message (COMMUNICATION_FAILD, errno);
@@ -579,7 +580,6 @@ PrimeConnection::send_command (const char *command,
             case EBADF:
             case EINVAL:
             case EPIPE:
-            case EIO:
                 clean_child ();
                 if (m_err_msg.empty ())
                     set_error_message (COMMUNICATION_FAILD, errno);
@@ -665,7 +665,6 @@ PrimeConnection::write_all (int fd, const char *buf, int size)
         case EBADF:
         case EINVAL:
         case EPIPE:
-        case EIO:
             return false;
         default:
             remaining -= rv;
