@@ -152,7 +152,7 @@
 #define SCIM_PRIME_CONFIG_SET_MODE_HALF_KATAKANA_KEY_DEFAULT  "F8"
 #define SCIM_PRIME_CONFIG_SET_MODE_WIDE_ASCII_KEY_DEFAULT     "F9"
 #define SCIM_PRIME_CONFIG_SET_MODE_RAW_KEY_DEFAULT            "F10"
-#define SCIM_PRIME_CONFIG_TOGGLE_ON_OFF_KEY_DEFAULT           "Zenkaku_Hankaku"
+#define SCIM_PRIME_CONFIG_TOGGLE_ON_OFF_KEY_DEFAULT           "Zenkaku_Hankaku,Control+j,Control+J"
 #define SCIM_PRIME_CONFIG_TOGGLE_LANGUAGE_KEY_DEFAULT         "F11"
 #define SCIM_PRIME_CONFIG_REGISTER_WORD_KEY_DEFAULT           "Control+w, Control+W"
 #define SCIM_PRIME_CONFIG_RECOVERY_KEY_DEFAULT                "Control+Alt+r"
